/*
 * Copyright (C) 2013 by the University of Southern California
 * $Id$
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 */

//#include "aesutil.hh"
#include <iostream>
#include <openssl/evp.h>
#include <cstring>
using namespace std;

/*
	AES decryption, in any cases, user should delete the returned string
*/
unsigned char *aec_decrypt(unsigned char *enctxt, int &len, unsigned char *key, unsigned char *iv)
{
	EVP_CIPHER_CTX *dp = EVP_CIPHER_CTX_new();
        if (!dp)
                return NULL;
	EVP_DecryptInit_ex(dp, EVP_aes_128_cbc(), NULL, key, iv);
	//unsigned char *cleartxt = new unsigned char[len + EVP_CIPHER_CTX_block_size(dp)];
	unsigned char *cleartxt = new unsigned char[len];
	//memset(cleartxt, '\0', len);
	int outlen = 0;
	int tmplen = 0;
        if(!EVP_DecryptUpdate(dp, cleartxt, &outlen, enctxt, len)) {
        	cerr << "[ERROR] aec_decrypt:EVP_DecryptUpdate fails." << endl;
                EVP_CIPHER_CTX_free(dp);
        	return NULL;
        }
        if(!EVP_DecryptFinal_ex(dp, cleartxt + outlen, &tmplen)) {
        	cerr << "[ERROR] aec_decrypt:EVP_DecryptFinal_ex fails." << endl;
        	EVP_CIPHER_CTX_free(dp);
        	return NULL;
        }	
        outlen += tmplen;
        len = outlen;
        EVP_CIPHER_CTX_free(dp);
	return cleartxt;
}

/*
	AES encryption, in any cases, user should delete the returned string
*/
unsigned char *aec_encrypt(unsigned char *cleartxt, int &len, unsigned char *key, unsigned char *iv)
{
	EVP_CIPHER_CTX *ep = EVP_CIPHER_CTX_new();
        if (!ep)
                return NULL;
	EVP_EncryptInit_ex(ep, EVP_aes_128_cbc(), NULL, key, iv);//NULL:no hardware
	unsigned char *enctxt = new unsigned char[len + EVP_CIPHER_CTX_block_size(ep)];
	//memset(enctxt, '\0', len + EVP_CIPHER_CTX_block_size(ep));
	int outlen = 0;
	int tmplen = 0;
        if(!EVP_EncryptUpdate(ep, enctxt, &outlen, cleartxt, len)) {
        	cerr << "[ERROR] aec_encrypt:EVP_EncryptUpdate fails." << endl;
        	EVP_CIPHER_CTX_free(ep);
        	return NULL;
        }

        if(!EVP_EncryptFinal_ex(ep, enctxt + outlen, &tmplen)) {
        	cerr << "[ERROR] aec_encrypt:EVP_EncryptFinal_ex fails." << endl;
        	EVP_CIPHER_CTX_free(ep);
        	return NULL;
        }
        outlen += tmplen;
        len = outlen;
        EVP_CIPHER_CTX_free(ep);
        return enctxt;
}

