/* -*-  Mode:C++; c-basic-offset:4; tab-width:4; indent-tabs-mode:t -*- */
/*
 * Copyright (C) 2016-2019 by the University of Southern California
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 */

/*
 * dsc_glue.cc
 *
 * dnsanon gluehook in and out of the dsc code
 *
 * This file is all USC code.
 */

#include <stdio.h>
#include <sys/types.h>
#include <netinet/tcp.h>
#include <libtrace.h>

#include <string.h>  // for memset
#include <string>

using namespace std;

extern "C" {
#include "dsc/src/dns_message.h"
#include "dsc/src/inX_addr.h"

	extern void tcpList_remove_older_than(long);
}
#include "dsc_glue.hh"
#include "parse.hh"
#include "utility.hh"

int debug_flag = 0;

static libtrace_packet_t *current_packet = 0;
static Parser *parser = 0;

void
dsc_register(Parser *p)
{
	if (parser)
		die("dsc_register: parser already registered");
	//xxx needs to register properly, this is not thread safe
	parser = p;
}

/*
 * dns_protocol_handler: from dsc into dnsanon
 *
 * buf is the actual data,
 * with len of it valid,
 * udata is a dsc "transport_message" with meta-information
 */
int
dns_protocol_handler(const u_char *buf, int len, void *udata)
{
	if (current_packet != parser->get_packet())
		die("dns_protocol_handler: packet changed in callback");
	transport_message *tm = reinterpret_cast<transport_message*>(udata);
	parser->per_message((u_char*)buf, len, tm);
	return 0;
}

void
inXaddr_assign_vX(inX_addr *dst, struct sockaddr *src)
{
	if (src->sa_family == AF_INET)
		(void)inXaddr_assign_v4(dst, &(((sockaddr_in *)src)->sin_addr));
	else if (src->sa_family == AF_INET6)
		(void)inXaddr_assign_v6(dst, &(((sockaddr_in6 *)src)->sin6_addr));
	else die("inXaddr_assign_vX: neither v4 nor v6\n");
}



/*
 * dnsanon_pcap_tcp_handler: from dnsanon into dsc
 */
void
dnsanon_pcap_tcp_handler(libtrace_packet_t *packet, libtrace_tcp_t *tcp_header, int len, u_char *payload)
{
	transport_message tm;
	memset(&tm, 0, sizeof(tm));
	tm.ts = trace_get_timeval(packet);

	struct sockaddr_storage addr_store;
	bzero(&addr_store, sizeof(struct sockaddr_storage));
	struct sockaddr *addr;

	addr = trace_get_source_address(packet, (struct sockaddr *)&addr_store);
	inXaddr_assign_vX(&tm.src_ip_addr, addr);
	addr = trace_get_destination_address(packet, (struct sockaddr *)&addr_store);
	inXaddr_assign_vX(&tm.dst_ip_addr, addr);
        tm.ip_version = addr->sa_family;

	tm.src_port = trace_get_source_port(packet); 
	tm.dst_port = trace_get_destination_port(packet);
	tm.proto = IPPROTO_TCP;

	/*
	 * (1) Sigh, can't round trip the packet through dsc, so we save it on the side.
	 * (2) We use whatever packet triggers a complete message as "THE" packet,
	 * so messages split over two packets get that ones timestamp.
	 */
	current_packet = packet;
	/* xxx: note that pcap_tcp_handler assumes the tcp data follows the header in memory. */
	(void)pcap_tcp_handler(reinterpret_cast<const struct tcphdr*>(tcp_header), len, (void*)&tm);
}

void
dnsanon_tcpList_remove_older_than(long t)
{
    tcpList_remove_older_than(t);
}
