/*
 * Copyright (C) 2013-2019 by the University of Southern California
 * $Id$
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 */
#ifndef PARSE_HH
#define PARSE_HH

#include <string>
#include <vector>
#include <libtrace.h>
#include <ldns/ldns.h>
#include "table.hh"

extern "C" {
#include "dsc/src/dns_message.h"
}

class Parser {
public:
    Parser(const std::string& in, const std::string& out, uint16_t, const std::string&, int);
    ~Parser();
    void parse();
    void per_message(u_char*, int, transport_message*);
    void set_write_emptyfiles(bool tf) { write_emptyfiles = tf; }
    void set_compressor(const std::string &cmd);
    void set_output_prefix(const std::string &pfix) { output_prefix = pfix; }
    //added for dsc_glue validation
    const libtrace_packet_t* get_packet() { return packet; }
protected:
    void do_section(ldns_pkt*, ldns_pkt_section, int, std::string, Table*, bool, std::string);
    void out_extra_info(const std::string&, std::string&, libtrace_tcp_t*, bool, Table*);
    void get_enc_key(const std::string&);
    void create_output(const std::string& timestamp);
    void per_packet(libtrace_packet_t*);

private:
    void clean_all_tables();
    
    static void clean_table(Table*&);
    static void sig_handler(int);
    static void libtrace_cleanup(libtrace_t *, libtrace_packet_t *);
    static void format_rdata(std::string &);
    static const std::string& determine_compressor_suffix(const std::string &);
    static uint8_t get_ip_ttl(libtrace_packet_t*, bool&);

private:
    //packet counter, just line number
    long long msgCount; 
    //the key and initialization vector; used for ip anonymization and domain name encryption
    unsigned char *enc_key, *enc_iv;
    int key_len, iv_len;
    //tables
    bool new_output;
    long output_count;
    uint16_t anon_flag;
    std::string inputPath, outputPath;
    int pbits;
    bool write_emptyfiles;
    std::string output_prefix;
    std::vector<std::string> compressor;

	libtrace_t *trace;
	libtrace_packet_t *packet;
    
    Table *PT, *QT, *RT, *TCPET, *UDPET, *RSSAC002T;
};
    
#endif// PARSE_HH
